import { applyPropsOnObjectExcept, safeJsonStringify } from "./utils";
import { nativeElementsPatches } from './native-elements-patches';
function getEventHandlerNames(wElement) {
    if (!wElement) {
        return [];
    }
    const keys = Object.keys(wElement);
    const eventHandlers = keys.filter(key => key.startsWith('on'));
    return eventHandlers;
}
function getInverseAction(action) {
    switch (action) {
        case 'hide':
            return 'show';
        case 'show':
            return 'hide';
        case 'collapse':
            return 'expand';
        case 'expand':
            return 'collapse';
    }
}
export class ReactVeloReconcilerInstance {
    constructor(props, log) {
        this._nativeEl = null;
        this.getIdentifier = () => this.props.id || this.type;
        this.relative$w = props.relative$w;
        this.instanceId = props.instanceId;
        this.type = props.type;
        this.props = props.props;
        this.rootContainer = props.rootContainer;
        this.hostContext = props.hostContext;
        this.children = props.children;
        this.parent = props.parent;
        this._log = log;
        this._eventHandlersRemovers = new Map();
    }
    _applyNativeElPatch(nativeEl) {
        const nativeType = nativeEl.type;
        const patch = nativeElementsPatches[nativeType];
        if (typeof patch === 'function') {
            patch(nativeEl);
        }
    }
    getNativeEl() {
        if (this._nativeEl) {
            return this._nativeEl;
        }
        const identifier = this.getIdentifier();
        const nativeEl = this.relative$w(`#${identifier}`);
        if (nativeEl) {
            this._nativeEl = nativeEl;
            this._applyNativeElPatch(this._nativeEl);
        }
        else {
            this._log(`Warning: no nativeEl for #${identifier} of type ${this.type} props id: ${this.props.id} on instanceId: ${this.instanceId}`);
        }
        return this._nativeEl;
    }
    toggleNativeInstanceVisibility(action) {
        const nativeEl = this.getNativeEl();
        const identifier = this.getIdentifier();
        if (nativeEl) {
            const inverseAction = getInverseAction(action);
            if (typeof nativeEl[action] === 'function' && typeof nativeEl[inverseAction] === 'function') {
                this._log(`innerToggleVisibility ${action} for instanceId: ${this.instanceId} propId: ${identifier}`);
                nativeEl[inverseAction]();
                nativeEl[action]();
            }
            else {
                this._log(`Warning: ${action}() is not defined for #${identifier} ${typeof this.props.id}}`);
            }
        }
        else {
            this._log(`Warning: ${action}() cannot be performed - element with id ${identifier} was not found`);
        }
    }
    toggleVisibility(action) {
        const identifier = this.getIdentifier();
        this._log(`toggleVisibility ${action} for instanceId: ${this.instanceId} propId: ${identifier}`);
        this.toggleNativeInstanceVisibility(action);
        this.children.map(child => child.toggleVisibility(action));
    }
    applyOnExistingEvents(fn) {
        const nativeElToInstallOn = this.getNativeEl();
        const eventHandlerNames = getEventHandlerNames(nativeElToInstallOn);
        const existingEvents = eventHandlerNames.filter(nativeEventName => typeof this.props[nativeEventName] === 'function');
        this._log(`Applying on existingEvents: (${existingEvents.join(',')}) on instanceId: ${this.instanceId} identifier: ${this.getIdentifier()}`);
        existingEvents.forEach(eventName => fn(eventName, nativeElToInstallOn));
    }
    installEventHandlers() {
        this.applyOnExistingEvents((eventName) => {
            this._addEventHandlerInstance(eventName, this.props[eventName]);
        });
    }
    removeEventHandlers() {
        this.applyOnExistingEvents((eventName, nativeElToRemoveFrom) => {
            nativeElToRemoveFrom.removeEventHandler(eventName, this.props[eventName]);
            this._removeEventHandlerByHandlerInstance(this.props[eventName]);
        });
    }
    _addEventHandlerInstance(eventName, eventHandler) {
        const eventHandlerRemover = this.getNativeEl()[eventName](eventHandler);
        if (typeof eventHandlerRemover === 'function') {
            this._eventHandlersRemovers.set(eventHandler, eventHandlerRemover);
        }
    }
    upsertEventHandler(eventName, newEventHandler, oldEventHandler) {
        if (oldEventHandler) {
            this.getNativeEl().removeEventHandler(eventName, oldEventHandler);
            this._removeEventHandlerByHandlerInstance(oldEventHandler);
        }
        this._addEventHandlerInstance(eventName, newEventHandler);
    }
    _removeEventHandlerByHandlerInstance(handler) {
        if (this._eventHandlersRemovers.has(handler)) {
            const remover = this._eventHandlersRemovers.get(handler);
            remover();
            this._eventHandlersRemovers.delete(handler);
        }
    }
    applyPropsOnNativeEl() {
        if (this.getNativeEl()) {
            const identifier = this.getIdentifier();
            const eventHandlerNames = getEventHandlerNames(this.getNativeEl());
            const unsettablePropNames = ['id', 'children', ...eventHandlerNames];
            this._log(`Setting props ${safeJsonStringify(this.props)} on nativeEl #${identifier}`);
            applyPropsOnObjectExcept(this.getNativeEl(), this.props, unsettablePropNames);
            this.applyVirtualPropsOnNativeEl();
        }
    }
    applyVirtualPropsOnNativeEl() {
        if (this.getNativeEl()) {
            if (typeof this.props['hidden'] !== 'undefined') {
                this.props['hidden'] ? this.getNativeEl().hide() : this.getNativeEl().show();
            }
        }
    }
    getEventHandlerNames() {
        return getEventHandlerNames(this.getNativeEl());
    }
    setRelative$w(relative$w) {
        this.relative$w = relative$w;
    }
    applyFunctionOnChildrenAndSelf(fn) {
        this.children.forEach(child => child.applyFunctionOnChildrenAndSelf(fn));
        fn(this);
    }
}
//# sourceMappingURL=reconciler-instance.js.map